package zero1hd.rhythmbullet.desktop.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.FrameBuffer;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.desktop.audio.MusicInfoController;
import zero1hd.rhythmbullet.desktop.audio.MusicList;
import zero1hd.rhythmbullet.desktop.audio.MusicListController;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.AnalysisPage;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.CreditsPage;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.KeybindOptionsPage;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.MainPage;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.MusicSelectionPage;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.OptionsPage;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.VideoOptionsPage;
import zero1hd.rhythmbullet.util.AdvancedResizeScreen;
import zero1hd.rhythmbullet.util.MusicManager;

public class MainMenu extends ScreenAdapter implements AdvancedResizeScreen {
	public Stage stage;
	private Vector3 cameraPosition;
	
	private MainPage mainPage;
	private OptionsPage optionsPage;
	private CreditsPage creditsPage;
	private KeybindOptionsPage keybindPage;
	private VideoOptionsPage graphicsPage;
	private MusicSelectionPage musicSelectionPage;
	private AnalysisPage analysisPage;
	private RhythmBullet core;
	
	private MusicListController mlc;
	private MusicInfoController mic;
	
	private float lerpAlpha;
	
	private ShaderProgram gaussianBlurShader;
	private ShaderProgram brightFilterShader;
	private ShaderProgram combineShader;
	private FrameBuffer lightFilterBuffer;
	private FrameBuffer normalBuffer;
	private FrameBuffer hBlur, vBlur;
	private TextureRegion fboRegion;
	private int fboSize;
	private int blurlvl;
	private Batch batch;
	private ScreenViewport screenViewport;
	
	public MainMenu(RhythmBullet core) {
		this.core = core;
		stage = new Stage(new ScreenViewport());
		cameraPosition = new Vector3(stage.getCamera().position);
		
		MusicList musicList = new MusicList();
		musicList.setSearchPath(core.getPrefs().getString("music dir"));
		musicList.asynchRefresh();
		mlc = new MusicListController(musicList, core.getPrefs());
		mlc.setAutoPlay(true);
		mlc.setShuffle(true);
		
		mic = new MusicInfoController(musicList);
	}
	
	@Override
	public void postAssetLoad() {
		attemptLoadShaders();
		mainPage = new MainPage(core, cameraPosition, mlc, this);
		mainPage.setPosition(0, 0);
		stage.addActor(mainPage);
		//End main menu
		
		keybindPage = new KeybindOptionsPage(core.getDefaultSkin(), core.getAssetManager(), cameraPosition);
		keybindPage.setPosition(-1f*Gdx.graphics.getWidth(), -1f*Gdx.graphics.getHeight());
		stage.addActor(keybindPage);
		
		graphicsPage = new VideoOptionsPage(core.getDefaultSkin(), core.getPrefs(), this, core.getAssetManager());
		
		graphicsPage.setPosition(-1f*Gdx.graphics.getWidth(), 1f*Gdx.graphics.getHeight());
		stage.addActor(graphicsPage);
		
		optionsPage = new OptionsPage(core, cameraPosition, keybindPage, mlc);
		optionsPage.setPosition(-Gdx.graphics.getWidth(), 0);
		stage.addActor(optionsPage);
		
		creditsPage = new CreditsPage(core.getDefaultSkin());
		creditsPage.setPosition(0, Gdx.graphics.getHeight());
		stage.addActor(creditsPage);
		
		analysisPage = new AnalysisPage(core.getDefaultSkin(),core.getAssetManager(), cameraPosition);
		analysisPage.setPosition(2*Gdx.graphics.getWidth(), 0f);
		stage.addActor(analysisPage);
		
		musicSelectionPage = new MusicSelectionPage(core.getDefaultSkin(), mlc, mic, core.getAssetManager(), cameraPosition, analysisPage);
		musicSelectionPage.setPosition(1f*Gdx.graphics.getWidth(), 0f);
		stage.addActor(musicSelectionPage);
		stage.addListener(new InputListener() {
			@Override
			public boolean keyUp(InputEvent event, int keycode) {
				if (keycode == Keys.ESCAPE) {
					stage.unfocusAll();
					keybindPage.unselect();
				}
				return super.keyUp(event, keycode);
			}
		});	
		
		stage.addListener(new ClickListener() {
			@Override
			public void clicked(InputEvent event, float x, float y) {
				if (stage.hit(x, y, true) == null) {
					stage.unfocusAll();
					keybindPage.unselect();
				}
				super.clicked(event, x, y);
			}
		});
		mlc.getMusicList().deleteObservers();
		mlc.deleteObservers();
		mlc.addObserver(musicSelectionPage);
		mlc.addObserver(mainPage);
		mlc.getMusicList().addObserver(mainPage);
		
		if (mlc.getMusicList().isSearched() && mlc.getCurrentMusicManager() != null) {
			MusicManager mManager = mlc.getCurrentMusicManager();
			mainPage.updateVisualsForDifferentSong(mManager);
			musicSelectionPage.refreshUIList();
		}
	}
	
	@Override
	public void render(float delta) {
		Gdx.gl.glClearColor(0.22f, 0.22f, 0.22f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		stage.act(delta);
		
		if (blurlvl > 0) {
//			Begin drawing a normal version of screen
			normalBuffer.begin();
			stage.getViewport().apply();
			Gdx.gl.glClearColor(0.2f, 0.2f, 0.2f, 1f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			draw();
			normalBuffer.end();

//			BEGINNING NORMAL SCREEN RENDER
			screenViewport.apply();
			
//			Begin light filtering
			lightFilterBuffer.begin();
			Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			fboRegion.setTexture(normalBuffer.getColorBufferTexture());
			batch.setShader(brightFilterShader);
			batch.setProjectionMatrix(screenViewport.getCamera().combined);
			batch.begin(); //BATCH STARTS HERE
			batch.draw(fboRegion, 0, 0, stage.getWidth(), stage.getHeight());
			batch.flush();
			lightFilterBuffer.end();
//			
			for (int i = 0; i < blurlvl; i++) {
//				Horizontal gaussian blur
				hBlur.begin();
				if (i > 0) {
					fboRegion.setTexture(vBlur.getColorBufferTexture());
				} else {
					fboRegion.setTexture(lightFilterBuffer.getColorBufferTexture());
				}
				batch.setShader(gaussianBlurShader);
				gaussianBlurShader.setUniformi("horizontal", 1);
				batch.draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
				batch.flush();
				hBlur.end();
				
//				//Vertical gaussian blur
				vBlur.begin();
				fboRegion.setTexture(hBlur.getColorBufferTexture());
				batch.setShader(gaussianBlurShader);
				gaussianBlurShader.setUniformi("horizontal", 0);
				batch.draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
				batch.flush();
				vBlur.end();
			}

			Gdx.gl.glClearColor(0f, 0f, 0f, 0f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			batch.setShader(combineShader);
			fboRegion.setTexture(normalBuffer.getColorBufferTexture());
			batch.draw(fboRegion, 0f, 0f, fboSize, fboSize);
			batch.setShader(null);
			batch.end(); //STAGE BATCH ENDS HERE

		} else {
			draw();
		}
        
		//move camera
		if (stage.getCamera().position.x != cameraPosition.x || stage.getCamera().position.y != cameraPosition.y) {
			stage.getCamera().position.lerp(cameraPosition, delta*lerpAlpha);
			stage.getViewport().apply();
		}
		
		super.render(delta);
	}
	
	private void draw() {
		//TODO draw background
		stage.draw();
	}
	
	@Override
	public void preAssetLoad() {
		stage.clear();
		mainPage.dispose();
		optionsPage.dispose();
		creditsPage.dispose();
		keybindPage.dispose();
		musicSelectionPage.dispose();
	}
	
	@Override
	public void show() {
		Gdx.input.setInputProcessor(stage);
		calcLerpAlpha(Gdx.graphics.getWidth());
		super.show();
	}
	
	@Override
	public void hide() {
		saveAll();
		super.hide();
	}
	
	public void saveAll() {
		if (optionsPage != null) {
			optionsPage.saveOptions(core.getPrefs());
			core.getPrefs().flush();
		}
	}
	
	@Override
	public void resize(int width, int height) {
		stage.getViewport().update(width, height, false);
		cameraPosition.x = width/2;
		cameraPosition.y = height/2;
		calcLerpAlpha(width);
		super.resize(width, height);
	}
	
	@Override
	public void dispose() {
		stage.dispose();
		unloadShaders();
		mic.dispose();
		super.dispose();
	}
	
	private void calcLerpAlpha(int width) {
		if (width <= 3835) {
			lerpAlpha = 5.0f;
		} else {
			lerpAlpha = 5.5f;
		}
	}
	
	public void unloadShaders() {
		if (core.getPrefs().getBoolean("glow shader")) {
			brightFilterShader.dispose();
			combineShader.dispose();
			gaussianBlurShader.dispose();
			normalBuffer.dispose();
			lightFilterBuffer.dispose();
			vBlur.dispose();
			hBlur.dispose();
		}
		
		brightFilterShader = null;
		combineShader = null;
		gaussianBlurShader = null;
		normalBuffer = null;
		lightFilterBuffer = null;
		vBlur = null;
		hBlur = null;
		
		setBlurlvl(0);
	}
	
	public void setBlurlvl(int blurlvl) {
		this.blurlvl = blurlvl;
	}
	
	public Vector3 getCameraPosition() {
		return cameraPosition;
	}

	public CreditsPage getCreditsPage() {
		return creditsPage;
	}
	
	public VideoOptionsPage getGraphicsPage() {
		return graphicsPage;
	}
	
	public KeybindOptionsPage getKeybindPage() {
		return keybindPage;
	}
	
	public MusicSelectionPage getMusicSelectionPage() {
		return musicSelectionPage;
	}
	
	public OptionsPage getOptionsPage() {
		return optionsPage;
	}
	
	public void attemptLoadShaders() {
		if (core.getPrefs().getBoolean("glow shader", true)) {
			if (core.getPrefs().getBoolean("enhanced glow", false)) {
				blurlvl = 5;
			} else {
				blurlvl = 1;
			}
			Gdx.app.debug("Shader", "Loading glow shaders.");
			batch = new SpriteBatch();
			screenViewport = new ScreenViewport();
			screenViewport.update(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
			((OrthographicCamera) screenViewport.getCamera()).setToOrtho(false, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
			
			Gdx.app.debug("Shader", "using glow shader");
			brightFilterShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/bright_filter.fsh"));
			if (!brightFilterShader.isCompiled()) {
				Gdx.app.error("Shader failed to compile", brightFilterShader.getLog());
				System.exit(0);
			}
			if (brightFilterShader.getLog().length() != 0) {
				Gdx.app.error("Shader", brightFilterShader.getLog());
			}
				
			gaussianBlurShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/gaussian_blur.fsh"));
			if (!gaussianBlurShader.isCompiled()) {
				Gdx.app.error("Shader failed to compile", gaussianBlurShader.getLog());
				System.exit(0);
			}
			if (gaussianBlurShader.getLog().length() != 0) {
				Gdx.app.error("Shader", gaussianBlurShader.getLog());
			}
			
			combineShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/combine.fsh"));
			if (!combineShader.isCompiled()) {
				Gdx.app.error("Shader failed to compile", combineShader.getLog());
				System.exit(0);
			}
			if (combineShader.getLog().length() != 0) {
				Gdx.app.error("Shader", combineShader.getLog());
			}
			
			
			if (Gdx.graphics.getWidth() < 1024) {
				fboSize = 1024;
			} else if (Gdx.graphics.getWidth() < 2048) {
				fboSize = 2048;
			} else {
				fboSize = 4096;
			}
			
			lightFilterBuffer = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);
			normalBuffer = new FrameBuffer(Format.RGBA8888, fboSize, fboSize, false);
			hBlur = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);
			vBlur = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);

			fboRegion = new TextureRegion(normalBuffer.getColorBufferTexture());
			fboRegion.flip(false, true);

			combineShader.begin();
			combineShader.setUniformi("u_texture1", 1);
			combineShader.end();
			
			vBlur.getColorBufferTexture().bind(1);
			
			Gdx.gl.glActiveTexture(GL20.GL_TEXTURE0);
		}
		
		ShaderProgram.pedantic = false;
	}

}
